<?php

/**
 * The file that defines the core plugin class
 *
 * A class definition that includes attributes and functions used across both the
 * public-facing side of the site and the admin area.
 *
 * @link       https://webappick.com
 * @since      1.0.0
 *
 * @package    Webappick_Pdf_Invoice_For_Woocommerce
 * @subpackage Webappick_Pdf_Invoice_For_Woocommerce/includes
 */

/**
 * The core plugin class that generate PDF Invoice.
 *
 * This is used to generate PDF Invoice
 *
 *
 * @since      1.0.0
 * @package    Webappick_Pdf_Invoice_For_Woocommerce
 * @subpackage Webappick_Pdf_Invoice_For_Woocommerce/includes
 * @author     Md Ohidul Islam <wahid@webappick.com>
 */

 class Webappick_Pdf_Invoice_Engine {

	/**
	 * The loader that's responsible for maintaining and registering all hooks that power
	 * the plugin.
	 *
	 * @since    1.0.0
	 * @access   public
	 * @var      Webappick_Pdf_Invoice_Engine $orderId store all order ids.
	 */
	
	public $orderIds;
	public $orderId;
	public $products;
	public $orderInfo;
	public $currencySymbol="";

    /**
     * Webappick_Pdf_Invoice_Engine constructor.
     * @param $orderIds
     */
    public function __construct($orderIds) {

		$this->orderIds=$orderIds;


        $this->products=array();
		$this->orderInfo=array();

		$this->extractOrdersInfo();

		if(!empty(get_option("wpifw_currency_code"))){
			$this->currencySymbol=" (".get_woocommerce_currency().")";
		}

	}

	/**
	 * Generate Bulk or Single PDF Invoice
	 */
	public function generatePDF( ) {

		$upload=wp_upload_dir();
		$baseDir=$upload['basedir'];

		ob_start();
		$orders=$this->orderInfo;
        # DEBUG
        # echo "<pre>";print_r(wc_get_product(128));die();

        # Get Template Content
        if($this->orderIds){
          echo $this->getTemplateContent('invoice',$orders);
        }
        else{
            require plugin_dir_path( __FILE__ ) . 'templates/empty.php';
        }
		$html = ob_get_contents();
		ob_end_clean();

		$defaultConfig = (new Mpdf\Config\ConfigVariables())->getDefaults();
		$fontDirs = $defaultConfig['fontDir'];

		$defaultFontConfig = (new Mpdf\Config\FontVariables())->getDefaults();
		$fontData = $defaultFontConfig['fontdata'];
		$mpdf = new \Mpdf\Mpdf([
			'tempDir' => $baseDir,
			'fontDir' => array_merge($fontDirs, [
				plugin_dir_path(__FILE__) . 'templates/fonts',
			]),
			'fontdata' => $fontData + [
					'Lato' => [
						'R' => "Lato-Regular.ttf",
						'B' => "Lato-Bold.ttf",
						'I' => "Lato-Italic.ttf",
						'BI' => "Lato-BoldItalic.ttf",
					],
                    'currencies' => [
                        'R' => "Currencies.ttf",
                    ]
				],
			'default_font' => 'Lato',
			'mode' => 'utf-8',
			'format'=>'A4'
		]);
		$mpdf->shrink_tables_to_fit=0;

		$mpdf->WriteHTML($html);
		$output=$mpdf->Output();
		exit;
	}


    /**
     * Generate Bulk or Single PDF Invoice Packing Slip
     */
    public function generatePackingSlip( ) {

        $upload=wp_upload_dir();
        $baseDir=$upload['basedir'];



        ob_start();
        $orders=$this->orderInfo;

        # DEBUG
        //echo "<pre>";print_r($orders);die();

        # Get Template Content
        if($this->orderIds){
            echo $this->getTemplateContent('slip',$orders);
        }
        else{
            require plugin_dir_path( __FILE__ ) . 'templates/empty.php';
        }
        $html = ob_get_contents();
        ob_end_clean();

        $defaultConfig = (new Mpdf\Config\ConfigVariables())->getDefaults();
        $fontDirs = $defaultConfig['fontDir'];

        $defaultFontConfig = (new Mpdf\Config\FontVariables())->getDefaults();
        $fontData = $defaultFontConfig['fontdata'];
        $mpdf = new \Mpdf\Mpdf([
            'tempDir' => $baseDir,
            'fontDir' => array_merge($fontDirs, [
                plugin_dir_path(__FILE__) . 'templates/fonts',
            ]),
            'fontdata' => $fontData + [
                    'Lato' => [
                        'R' => "Lato-Regular.ttf",
                        'B' => "Lato-Bold.ttf",
                        'I' => "Lato-Italic.ttf",
                        'BI' => "Lato-BoldItalic.ttf",
                    ]
                ],
            'default_font' => 'Lato',
            'mode' => 'utf-8',
            'format'=>'A4'
        ]);
        $mpdf->shrink_tables_to_fit=0;

        $mpdf->WriteHTML($html);
        $output=$mpdf->Output();
        exit;
    }


    /**
     * Extract all order information and store as array
     */
    public function extractOrdersInfo(  ) {
        $orderIds=explode(",",$this->orderIds);
        foreach ($orderIds as $order_key=>$order_id){

            # Initialize order information into array
            $order = wc_get_order($order_id);


            if($order){
                $shipping_method = $order->get_shipping_method();
                $payment_method = $order->get_payment_method_title();
                $order_data=$order->get_data();
                foreach ($order_data as $order_key=>$order_value){
                    $this->orderInfo[$order_id][$order_key]=$order_value;
                }


                # Set Order Date according to plugin settings
                $date=$order->get_date_completed();
                if(!is_object($date)){
                    $date=$order->get_date_created();
                }
                $this->orderInfo[$order_id]['wpifw_order_date']=$date->date("d M, o");


                # Get Products by Order Id
                foreach ($order->get_items() as $item_key => $item_values):

                    $item_id = $item_values->get_id();
                    $product = $item_values->get_product();
                    $product_data=$product->get_data();
                    $item_data = $item_values->get_data();


                    foreach ($item_data as $key=>$value){
                        $this->products[$order_id][$item_id][$key]=$value;
                    }

                    $this->products[$order_id][$item_id]['rrrr']=wc_get_order_item_meta($item_id,"_line_subtotal_tax",true);
                    $this->products[$order_id][$item_id]['pid']=$product_data['id'];
                    $this->products[$order_id][$item_id]['sku']=$product_data['sku'];
                    $this->products[$order_id][$item_id]['product_info']=wc_get_product($product_data['id']);

                endforeach;

                $this->orderInfo[$order_id]['wpifw_products']       =    $this->products[$order_id];
                $this->orderInfo[$order_id]['all_products']         =    $this->products;
                $this->orderInfo[$order_id]['shipping_method']      =    $shipping_method;
                $this->orderInfo[$order_id]['payment_method_title'] =    $payment_method;

            }
        }

    }


    /** Make Invoice / Packing Slip Content from Template
     * @param $type
     * @param $orders
     * @return bool|mixed|string
     */

    public function getTemplateContent($type, $orders)
    {
        if($type=="invoice"){
            # Get Invoice Content
            $template=file_get_contents(plugin_dir_path(__FILE__) .'templates/invoice.php');

        }elseif ($type=="slip"){
            # Get Invoice Slip Content
            $template=file_get_contents(plugin_dir_path(__FILE__) .'templates/invoice_packing_slip.php');
        }


        # Get Template Header
        $content= $this->get_string_between($template,"<=head-start=>","<=head-end=>",array("{{PACKING_SLIP_TEXT}}",'{{INVOICE}}'),array("PACKING SLIP",'INVOICE'));

        # Calculate Page Break
        $tOrder=count($orders);
        $or=1;
        $pageBreak="pageBreak";

        foreach ($orders as $order_id=>$order){


            # Process Page Break
            if($tOrder==$or){
                $pageBreak="";
            }
            $or++;

            # Get Template Body
            $body= $this->get_string_between($template,"<=body-top-start=>","<=body-top-end=>",array("{{pagebreak}}",'$order_id'),array($pageBreak,$order_id));
            # Display Product Information

            if(isset($order['wpifw_products'])){

                foreach ($order['wpifw_products'] as $key=>$value) {

                    $p=0;
                    # TODO Make new page if products more than 6
//                        if ($p == 1) {
//                            break;
//                        }
                    $id = $value['product_id'];
                    $body .= $this->get_string_between($template, "<=product-loop-start=>", "<=product-loop-end=>",'$id',$id);
                }
            }

            $body.= $this->get_string_between($template,"<=body-bottom-start=>","<=body-bottom-end=>",'$order_id',$order_id);

            if(!get_option('wpifw_payment_method_show')) {
                $body= $this->remove_string_between_tax_discount($body, "<=body-top-invoice_payment_start=>", "<=body-top-invoice_payment_end=>");
            }
            if(!$order['total_tax']) {
                $body= $this->remove_string_between_tax_discount($body, "<=body-remove-tax-start=>", "<=body-remove-tax-end=>");
            }
            if(!$order['discount_total']) {
                $body= $this->remove_string_between_tax_discount($body, "<=body-remove-discount-start=>", "<=body-remove-discount-end=>");
            }
            if(!$order['shipping_total'] || $order['shipping_total'] == 0.00) {
                $body= $this->remove_string_between_tax_discount($body, "<=body-remove-shipping-start=>", "<=body-remove-shipping-end=>");
            }

            $content.=$this->replaceOrderString($order,$body);
        }

        # Get Template Footer
        $content.= $this->get_string_between($template,"<=footer-start=>","<=footer-end=>");

        return $content;
    }

    /**
     * Get Template Strings between Template Codes
     *
     * @param $string
     * @param string $start Template code to start
     * @param string $end Template Code to end
     * @param string $toReplace Order or Product id to replace
     * @param string $replaceWith  Order or Product id replace with
     * @return bool|mixed|string
     */
    public function get_string_between($string, $start, $end, $toReplace="", $replaceWith=""){
        $string = ' ' . $string;
        $ini = strpos($string, $start);
        if ($ini == 0) return '';
        $ini += strlen($start);
        $len = strpos($string, $end, $ini) - $ini;
        $finalString=substr($string, $ini, $len);
        $finalString=str_replace($toReplace,$replaceWith,$finalString);
        return $finalString;
    }
    public function remove_string_between_tax_discount($string, $start, $end){
        $beginningPos = strpos($string, $start);
        $endPos = strpos($string, $end);
        if ($beginningPos === false || $endPos === false) {
            return $string;
        }

        $textToDelete = substr($string, $beginningPos, ($endPos + strlen($end)) - $beginningPos);
        $string =  str_replace($textToDelete, '', $string);
        return $string;
    }

    /**
	 * Find and Replace all dynamic string for Invoice Template
	 *
	 * @param $order
	 * @param $string
	 *
	 * @return mixed
	 */
	public function replaceOrderString($order,$string) {

		$id=$order['id'];
		$INVOICE_NUMBER_TEXT=get_option('wpifw_INVOICE_NUMBER_TEXT');
		$INVOICE_DATE_TEXT=get_option('wpifw_INVOICE_DATE_TEXT');
		$INVOICE_TITLE    =get_option('wpifw_INVOICE_TITLE');
		$ORDER_NUMBER_TEXT=get_option('wpifw_ORDER_NUMBER_TEXT');
		$ORDER_DATE_TEXT=get_option('wpifw_ORDER_DATE_TEXT');
		$SL=get_option('wpifw_SL');
		$PRODUCT=get_option('wpifw_PRODUCT');
		$PRICE=get_option('wpifw_PRICE');
		$QUANTITY=get_option('wpifw_QUANTITY');
		$ROW_TOTAL=get_option('wpifw_ROW_TOTAL');
		$SUBTOTAL_TEXT=get_option('wpifw_SUBTOTAL_TEXT');
		$TAX_TEXT=get_option('wpifw_TAX_TEXT');
		$DISCOUNT_TEXT=get_option('wpifw_DISCOUNT_TEXT');
        $SHIPPING_TEXT=get_option('wpifw_SHIPPING_TEXT');
		$GRAND_TOTAL_TEXT=get_option('wpifw_GRAND_TOTAL_TEXT');
		$LOGO_HEIGHT= get_option('wpifw_wpifw_logo_height');
		$LOGO_WIDTH = get_option('wpifw_logo_width');
		$TERMS_AND_CONDITION = get_option('wpifw_terms_and_condition');
		$OTHER_INFORMATION = get_option('wpifw_other_information');

		#For Packing slip localization setting setup
        $PACKING_SLIP_TEXT = get_option('wpifw_PACKING_SLIP_TEXT');
        $PACKING_SLIP_ORDER_NUMBER_TEXT = get_option('wpifw_PACKING_SLIP_ORDER_NUMBER_TEXT');
        $PACKING_SLIP_ORDER_DATE_TEXT   = get_option('wpifw_PACKING_SLIP_ORDER_DATE_TEXT');
        $PACKING_SLIP_ORDER_METHOD_TEXT = get_option('wpifw_PACKING_SLIP_ORDER_METHOD_TEXT');
        $PACKING_SLIP_PRODUCT_TEXT      = get_option('wpifw_PACKING_SLIP_PRODUCT_TEXT');
        $PACKING_SLIP_WEIGHT_TEXT       = get_option('wpifw_PACKING_SLIP_WEIGHT_TEXT');
        $PACKING_SLIP_QUANTITY_TEXT     = get_option('wpifw_PACKING_SLIP_QUANTITY_TEXT');
        $PAYMENT_METHOD_TEXT            = get_option('wpifw_payment_method_text');
		#Product Information To Replace
		$prodToReplace=array();
		#Product Information Replace With
		$prodReplaceWith=array();
		$subtotal = 0;
		foreach ($order['wpifw_products'] as $p_key=>$p_value){

			array_push($prodToReplace,"{{P_SL_".$p_value['product_id']."}}");
			array_push($prodToReplace,"{{P_DESCRIPTION_".$p_value['product_id']."}}");
			array_push($prodToReplace,"{{P_PRICE_".$p_value['product_id']."}}");
			array_push($prodToReplace,"{{P_QUANTITY_".$p_value['product_id']."}}");
			array_push($prodToReplace,"{{P_TOTAL_".$p_value['product_id']."}}");
            array_push($prodToReplace,"{{WEIGHT_".$p_value['product_id']."}}");
            $product            = $p_value['product_info'];
//            $individualSubtotal = $product->get_price() * $p_value['quantity'];
            $subtotal           = $subtotal + $p_value['subtotal'];
			array_push($prodReplaceWith,$p_value['product_id']);
			array_push($prodReplaceWith,$this->processProductInfo($p_value['name'],$p_value['sku'],$p_value['product_id']));
			array_push($prodReplaceWith,$this->formatPrice($product->get_price()));
			array_push($prodReplaceWith,$p_value['quantity']);
			array_push($prodReplaceWith,$this->formatPrice($p_value['subtotal']));
			array_push($prodReplaceWith,$this->getProductWeight($product));

		}
#If tax and discount are not available ,the subtotal will be the grandTotal
        $grandTotal = $subtotal;

		#Assign shipping method for multiple bulk action

		if(isset($order['shipping_method'])){
			foreach ($order['shipping_method'] as $s_key=>$s_value){
				array_push($prodToReplace, "{{ORDER_METHOD".$s_value['shipping_method']."}}");
				array_push($prodReplaceWith, $s_value['shipping_method']);
			}
		}




        $textToReplace=array(
			"{{LOGO}}",
			"{{TO_$id}}",
			"{{FROM}}",
			"{{INVOICE_NUMBER_TEXT}}",
			"{{INVOICE_DATE_TEXT}}",
			"{{ORDER_NUMBER_TEXT}}",
			"{{ORDER_DATE_TEXT}}",
			"{{SL}}",
			"{{PRODUCT}}",
			"{{PRICE}}",
			"{{QUANTITY}}",
			"{{ROW_TOTAL}}",
			"{{SUBTOTAL_TEXT}}",
			"{{TAX_TEXT}}",
			"{{DISCOUNT_TEXT}}",
			"{{SHIPPING_TEXT}}",
			"{{GRAND_TOTAL_TEXT}}",
			"{{ORDER_NUMBER_$id}}",
			"{{ORDER_DATE_$id}}",
			"{{INVOICE_NUMBER_$id}}",
			"{{INVOICE_DATE_$id}}",
			"{{SUBTOTAL_$id}}",
            "{{TAX_$id}}",
			"{{DISCOUNT_$id}}",
			"{{SHIPPING_$id}}",
			"{{GRAND_TOTAL_$id}}",
			"{{CURRENCY}}",
            "{{ORDER_METHOD_$id}}",
            "{{WEIGHT}}",
            "{{WEIGHT_$id}}",
            "{{INVOICE}}",
            "{{PACKING_SLIP_TEXT}}",
            "{{PACKING_SLIP_ORDER_NUMBER_TEXT}}",
            "{{PACKING_SLIP_ORDER_DATE_TEXT}}",
            "{{PACKING_SLIP_ORDER_METHOD_TEXT}}",
            "{{PACKING_SLIP_PRODUCT_TEXT}}",
             "{{PACKING_SLIP_WEIGHT_TEXT}}",
            "{{PACKING_SLIP_QUANTITY_TEXT}}",
            "{{LOGO_HEIGHT}}",
            "{{LOGO_WIDTH}}",
            "{{TERMS_AND_CONDITION}}",
            "{{OTHER_INFORMATION}}",
            "{{INVOICE_PAYMENT_METHOD_TITLE_TEXT}}",
            "{{INVOICE_PAYMENT_METHOD_TITLE_TEXT_$id}}",


		);

		$textReplaceWith=array(
			$this->resizeStoreLogo(),
			$this->processBillingAddress($order),
			$this->SellerInfo(),
			(!empty($INVOICE_NUMBER_TEXT)?$INVOICE_NUMBER_TEXT:"INVOICE NUMBER"),
			(!empty($INVOICE_DATE_TEXT)?$INVOICE_DATE_TEXT:"INVOICE DATE"),
			(!empty($ORDER_NUMBER_TEXT)?$ORDER_NUMBER_TEXT:"ORDER NUMBER"),
			(!empty($ORDER_DATE_TEXT)?$ORDER_DATE_TEXT:"ORDER DATE"),
			(!empty($SL)?$SL:"SL"),
			(!empty($PRODUCT)?$PRODUCT:"PRODUCTS"),
			(!empty($PRICE)?$PRICE:"PRICE"),
			(!empty($QUANTITY)?$QUANTITY:"QUANTITY"),
			(!empty($ROW_TOTAL)?$ROW_TOTAL:"TOTAL"),
			(!empty($SUBTOTAL_TEXT)?$SUBTOTAL_TEXT:"SUB TOTAL"),
            (!empty($TAX_TEXT)?$TAX_TEXT:"TAX"),
            (!empty($DISCOUNT_TEXT)?$DISCOUNT_TEXT:"DISCOUNT"),
            (!empty($SHIPPING_TEXT)?$SHIPPING_TEXT:"SHIPPING"),
			(!empty($GRAND_TOTAL_TEXT)?$GRAND_TOTAL_TEXT:"TOTAL"),
			$id,
			$this->processOrderDate($order),
			get_post_meta($id,"wpifw_invoice_no",true),
			$this->processOrderDate($order),
			$this->formatPrice($subtotal),
            (!empty($order['total_tax']))?$this->formatPrice($order['total_tax']):"",
            (!empty($order['discount_total']))?$this->formatPrice($order['discount_total']):"",
            (!empty($order['shipping_total']))?$this->formatPrice($order['shipping_total']):"",
			$this->formatPrice($order['total']),
			$this->currencySymbol,
            $order['shipping_method'],
            "WEIGHT",
            "WEIGHT_$id",
            (!empty($INVOICE_TITLE)?$INVOICE_TITLE:"INVOICE"),
            (!empty($PACKING_SLIP_TEXT)?$PACKING_SLIP_TEXT:"PACKING SLIP"),
            (!empty($PACKING_SLIP_ORDER_NUMBER_TEXT)?$PACKING_SLIP_ORDER_NUMBER_TEXT:"ORDER NUMBER"),
            (!empty($PACKING_SLIP_ORDER_DATE_TEXT)?$PACKING_SLIP_ORDER_DATE_TEXT:"ORDER DATE"),
            (!empty($PACKING_SLIP_ORDER_METHOD_TEXT)?$PACKING_SLIP_ORDER_METHOD_TEXT:"SHIPPING METHOD"),
            (!empty($PACKING_SLIP_PRODUCT_TEXT)?$PACKING_SLIP_PRODUCT_TEXT:"PRODUCT"),
            (!empty($PACKING_SLIP_WEIGHT_TEXT)?$PACKING_SLIP_WEIGHT_TEXT:"WEIGHT"),
            (!empty($PACKING_SLIP_QUANTITY_TEXT)?$PACKING_SLIP_QUANTITY_TEXT:"QUANTITY"),
            (!empty($LOGO_HEIGHT))? $LOGO_HEIGHT:"200",
            (!empty($LOGO_WIDTH))? $LOGO_WIDTH:"100",
            (!empty($TERMS_AND_CONDITION))? $TERMS_AND_CONDITION:"",
            (!empty($OTHER_INFORMATION))? $OTHER_INFORMATION:"",
            (!empty($PAYMENT_METHOD_TEXT))? $PAYMENT_METHOD_TEXT:"PAYMENT METHOD",
            (!empty($order['payment_method_title']))? $order['payment_method_title']:"",

		);
//        # For DEBUG
//        echo "<pre>";
//        print_r($textToReplace);
//        print_r($textReplaceWith);
//        print_r($prodToReplace);
//        print_r($prodReplaceWith);
//        die();

		# Replace Order Information
		$html=str_replace($textToReplace,$textReplaceWith,$string);
		# Replace Product Information
		$html=str_replace($prodToReplace,$prodReplaceWith,$html);

		return $html;
	}


    /**
     * Get Individual product weight for making invoice/ invoice slip
     * @param $product
     * @return string
     */
    public function getProductWeight($product){

        $weight=$product->get_weight();
        $weight = wc_format_weight($weight);
        return $weight;
    }
	/**
	 * Format product price with currency symbol
	 *
	 * @param $price
	 *
	 * @return string
	 */
	public function formatPrice($price) {

        #DEBUG

		if(!empty(get_option("wpifw_currency_symbol"))){
			return wc_price($price);
		}else{
			return $price;
		}
	}


	/**
	 * Save PDF Invoice to Upload directory
	 * @param $invoiceId
	 * @return string
	 */
	public function savePdf($invoiceId) {

		$upload  = wp_upload_dir();

		$baseDir = $upload['basedir'];

		ob_start();
		$orders = $this->orderInfo;
		if($orders){
        echo $this->getTemplateContent('invoice',$orders);
        }
		$html = ob_get_contents();
		ob_end_clean();

		$defaultConfig = (new Mpdf\Config\ConfigVariables())->getDefaults();
		$fontDirs = $defaultConfig['fontDir'];

		$defaultFontConfig = (new Mpdf\Config\FontVariables())->getDefaults();
		$fontData = $defaultFontConfig['fontdata'];
		$mpdf = new \Mpdf\Mpdf([
			'tempDir' => $baseDir,
			'fontDir' => array_merge($fontDirs, [
				plugin_dir_path(__FILE__) . 'templates/fonts',
			]),
			'fontdata' => $fontData + [
					'Lato' => [
						'R' => "Lato-Regular.ttf",
						'B' => "Lato-Bold.ttf",
						'I' => "Lato-Italic.ttf",
						'BI' => "Lato-BoldItalic.ttf",
					]
				],
			'default_font' => 'Lato',
			'mode' => 'utf-8',
			'format'=>'A4'
		]);
		$mpdf->shrink_tables_to_fit=0;

		$mpdf->WriteHTML($html);

		return $mpdf->Output(WPIFW_INVOICE_DIR."Invoice-".$invoiceId.".pdf","F");
	}

	/**
	 * Set date format according to plugin settings
	 */
	public function processOrderDate( $order) {
		#TODO get plugins date format settings
		$date=$order['date_created'];

		$format="d M, o";
		$getFormat=get_option("wpifw_date_format");
		if(!empty($getFormat)){
			$format=$getFormat;
		}
		return  $date->date($format);

	}

	/**
	 * Set billing address according to plugin settings
	 *
	 * @param $order
	 *
	 * @return string
	 */
	public function processBillingAddress($order) {

		$details=esc_textarea(get_option("wpifw_buyer"));

		$toReplace=array(
			"{{first_name}}",
			"{{last_name}}",
			"{{company}}",
			"{{address_1}}",
			"{{address_2}}",
			"{{city}}",
			"{{state}}",
			"{{postcode}}",
			"{{country}}",
			"{{phone}}",
			"{{email}}",
			"\n",
		);
		$replaceWith=array(
			!empty($order['billing']['first_name'])?$order['billing']['first_name']:"==",
			!empty($order['billing']['last_name'])?$order['billing']['last_name']:"==",
			!empty($order['billing']['company'])?$order['billing']['company']:"==",
			!empty($order['billing']['address_1'])?$order['billing']['address_1']:"==",
			!empty($order['billing']['address_2'])?$order['billing']['address_2']:"==",
			!empty($order['billing']['city'])?$order['billing']['city']:"==",
			!empty($order['billing']['state'])?$order['billing']['state']:"==",
			!empty($order['billing']['postcode'])?$order['billing']['postcode']:"==",
			!empty($order['billing']['country'])?$order['billing']['country']:"==",
			!empty($order['billing']['phone'])?$order['billing']['phone']:"==",
			!empty($order['billing']['email'])?$order['billing']['email']:"==",
			"<br>",
		);


		$to=get_option("wpifw_block_title_to");

		# Replace Billing information according to customers settings
		$address=str_replace($toReplace,$replaceWith,$details);
		# Remove Empty Line
		$address2=str_replace("<br>==","",$address);
		return "<b>".$to."</b><br>".$address2;
	}

	/**
	 * Format Product Name
	 * @param $name
	 * @param $sku
	 * @param $id
	 */
	public function processProductInfo($name,$sku,$id ) {
		$pInfo=$name;

		$displayInfo=get_option("wpifw_disid");
		if(!empty($displayInfo)){
			if($displayInfo=="ID"){
				$pInfo.="<br/><br/><span  style='font-size: x-small;color: #525659;'>ID: $id</span>";
			}elseif ($displayInfo=="SKU"){
			    if(!empty($sku)){
				$pInfo.="<br/><br/><span  style='font-size: x-small;color: #525659;'>SKU: ".strtoupper(strtolower($sku))."</span>";
                }

			}
		}

		return $pInfo;
	}

	/**
	 * Set shipping address according to plugin settings
	 */
	public function processShippingAddress( ) {

	}

	/**
	 * Set invoice number according to plugin settings
	 */
	public function processInvoiceNumber(  ) {

	}

	/**
	 * Set order number according to plugin settings
	 */
	public function processOrderNumber(  ) {

	}

	/**
	 * Resize store logo according to plugin settings
	 */
	public function resizeStoreLogo(  ) {
		return wp_get_attachment_url( get_option( 'wpifw_logo_attachment_id' ) );//get_option("wpifw_store_logo");
	}

	/**
	 * Set store logo according to plugin settings
	 */
	public function invoiceText(  ) {

	}

	/**
	 * Set product Information according to plugin settings
	 */
	public function productInformation(  ) {

	}

	/**
	 * Customer Info according to plugin settings
	 */
	public function BuyerInfo(  ) {

	}

	/**
	 * Seller Info according to plugin settings
	 */
	public function SellerInfo( ) {
		$from=get_option("wpifw_block_title_from");
		$company=get_option("wpifw_cname");
		$address=str_replace("\n","<br>",esc_textarea(get_option("wpifw_cdetails")));

		return "<b>$from</b><br>$company<br>$address";
	}

	/**
	 * Set localization value according to plugin settings
	 */
	public function localization(  ) {

	}


}

# Initialize Invoice Engine
function WPIFW_PDF($orderIds){
	return $invoice = new Webappick_Pdf_Invoice_Engine($orderIds);
}