<?php
/*
Plugin Name:  Woo Invoice (PDF Invoice and Packing Slip)
Plugin URI:   https://webappick.com
Description:  Automatic Generate PDF Invoice and attach  with order email for WooCommerce.
Version:      1.0.10
Author:       WebAppick
Author URI:   https://webappick.com
License:      GPL2
License URI:  https://www.gnu.org/licenses/gpl-2.0.html
Text Domain:  webappick-pdf-invoice-for-woocommerce
Domain Path:  /languages
WC requires at least: 3.2
WC tested up to: 3.4
*/


// If this file is called directly, abort.
if ( ! defined( 'WPINC' ) ) {
	die;
}

/**
 * Currently plugin version.
 * Start at version 1.0.0 and use SemVer - https://semver.org
 * Rename this for your plugin and update it as you release new versions.
 */
define( 'WAPDFINVOICE_VERSION', '1.0.10' );

/**
 * The code that runs during plugin activation.
 * This action is documented in includes/class-webappick-pdf-invoice-for-woocommerce-activator.php
 */
function activate_webappick_pdf_invoice_for_woocommerce() {
	require_once plugin_dir_path( __FILE__ ) . 'includes/class-webappick-pdf-invoice-for-woocommerce-activator.php';
	Webappick_Pdf_Invoice_For_Woocommerce_Activator::activate();
}

/**
 * The code that runs during plugin deactivation.
 * This action is documented in includes/class-webappick-pdf-invoice-for-woocommerce-deactivator.php
 */
function deactivate_webappick_pdf_invoice_for_woocommerce() {
	require_once plugin_dir_path( __FILE__ ) . 'includes/class-webappick-pdf-invoice-for-woocommerce-deactivator.php';
	Webappick_Pdf_Invoice_For_Woocommerce_Deactivator::deactivate();
}

register_activation_hook( __FILE__, 'activate_webappick_pdf_invoice_for_woocommerce' );
register_deactivation_hook( __FILE__, 'deactivate_webappick_pdf_invoice_for_woocommerce' );

/**
 * The core plugin class that is used to define internationalization,
 * admin-specific hooks, and public-facing site hooks.
 */
require plugin_dir_path( __FILE__ ) . 'includes/class-webappick-pdf-invoice-for-woocommerce.php';

/**
 * Begins execution of the plugin.
 *
 * Since everything within the plugin is registered via hooks,
 * then kicking off the plugin from this point in the file does
 * not affect the page life cycle.
 *
 * @since    1.0.0
 */
function run_webappick_pdf_invoice_for_woocommerce() {

	$plugin = new Webappick_Pdf_Invoice_For_Woocommerce();
	$plugin->run();

}
run_webappick_pdf_invoice_for_woocommerce();



/**
 * Register PDF Invoice Menu.
 */
function webappick_wpifw_main_menu_page() {
	add_menu_page(__('Woo Invoice', 'webappick-pdf-invoice-for-woocommerce'), __('Woo Invoice', 'webappick-pdf-invoice-for-woocommerce'), 'manage_options', __FILE__, 'webappick_wpifw_pdf_invoice', 'dashicons-media-spreadsheet');
	add_submenu_page(__FILE__, __('Settings', 'webappick-pdf-invoice-for-woocommerce'), __('Settings', 'webappick-pdf-invoice-for-woocommerce'), 'manage_options', __FILE__, 'webappick_wpifw_pdf_invoice');
//	add_submenu_page(__FILE__, __('Premium', 'webappick-pdf-invoice-for-woocommerce'), __('Premium', 'webappick-pdf-invoice-for-woocommerce'), 'manage_options', 'woo_feed_manage_feed', 'woo_feed_manage_feed');
}
add_action( 'admin_menu', 'webappick_wpifw_main_menu_page' );


/**
 *  Load Plugin settings page, process and save setting
 */
function webappick_wpifw_pdf_invoice(){

    $invoiceAllow = "wpifw_invoicing";
    $emailAlow = "wpifw_order_email";
    $downloadAllow = "wpifw_download";
    $currencyAllow = "wpifw_currency_code";
    $paymentmethod = "wpifw_payment_method_show";
    # Process settings form data and update
	if(isset($_POST['wpifw_submit'])){
        #If not checkbox is not checked then put empty value
        if(!isset($_POST[$invoiceAllow]) || !isset($_POST[$emailAlow]) || !isset($_POST[$downloadAllow]) || !isset($_POST[$currencyAllow]) || !isset($_POST[$paymentmethod])){
            update_option($invoiceAllow,sanitize_textarea_field(""));
            update_option($emailAlow,sanitize_textarea_field(""));
            update_option($downloadAllow,sanitize_textarea_field(""));
            update_option($currencyAllow,sanitize_text_field(""));
            update_option($paymentmethod,sanitize_text_field(""));
        }
		foreach($_POST as $key=>$value){
				update_option($key,sanitize_text_field($value));
		}
	}

	#Seller&Buyer form submitted, update the option settings
    if(isset($_POST['wpifw_submit_seller&buyer'])){
            foreach($_POST as $key=>$value){
                if($key=="wpifw_buyer" || $key=="wpifw_cdetails" || $key == "wpifw_terms_and_condition" || $key == "wpifw_other_information"){
                    update_option($key,sanitize_textarea_field($value));
                }else{
                    update_option($key,sanitize_text_field($value));
                }
            }
        }
#Localization form submitted, update the option settings
	if(isset($_POST['wpifw_submit_localization'])){
	    foreach($_POST as $key => $value){
            update_option($key,sanitize_text_field(strtoupper($value)));
        }
    }
#Batch Download Form submitting checked started
    if(isset($_POST['wpifw_submit_bulk_download'])){

        $args = array(
            'date_created' => $_POST['wpifw_date_from'] . '...' . $_POST['wpifw_date_to'],
            'return' => 'ids'
        );
        $orderIds = wc_get_orders( $args );

        $orderIds = implode(',',$orderIds);
#Bulk type checked and downloads the invoice and slip between the input dates
       if($_POST['wpifw_bulk_type'] == 'WPIFW_INVOICE_DOWNLOAD'){

           header("location:".htmlspecialchars_decode(wp_nonce_url( admin_url( 'admin-ajax.php?action=wpifw_generate_invoice&order_id=' . $orderIds ), 'webappick-pdf-invoice-for-woocommerce' )));

       }else if($_POST['wpifw_bulk_type'] == 'WPIFW_PACKING_SLIP' ){

           header("location:".htmlspecialchars_decode(wp_nonce_url( admin_url( 'admin-ajax.php?action=wpifw_generate_invoice_packing_slip&order_id=' . $orderIds ), 'webappick-pdf-invoice-for-woocommerce' )));
       }

    }
#Batch Download Form submitting checked ended

	# Load plugin settings view
	require plugin_dir_path(__FILE__) . 'admin/partials/webappick-pdf-invoice-for-woocommerce-admin-display.php';
}



/**
 * Process PDF Invoice Making Action
 */
add_action('wp_ajax_wpifw_generate_invoice', 'webappick_wpifw_generate_invoice');
function webappick_wpifw_generate_invoice($orderIds)
{
	if(isset($_REQUEST['order_id'])){
		$orderIds=sanitize_text_field($_REQUEST['order_id']);
    }elseif (isset($_REQUEST['order_ids'])){
		$orderIds=sanitize_text_field($_REQUEST['order_ids']);
    }

	WPIFW_PDF($orderIds)->generatePDF();

}

/**
 * Process PDF Packing slip Making Action
 */
add_action('wp_ajax_wpifw_generate_invoice_packing_slip', 'webappick_wpifw_generate_invoice_packing_slip');
function webappick_wpifw_generate_invoice_packing_slip($orderIds)
{

    if(isset($_REQUEST['order_id'])){
        $orderIds=sanitize_text_field($_REQUEST['order_id']);
    }elseif (isset($_REQUEST['order_ids'])){
        $orderIds=sanitize_text_field($_REQUEST['order_ids']);
    }


    WPIFW_PDF($orderIds)->generatePackingSlip();

}


add_action( 'admin_notices', 'woo_pdf_review_notice' );

function woo_pdf_review_notice() {
	$options = get_option('woo_pdf_review_notice');

	if(!$options){
		$notice = '<div class="woo-pdf-review-notice notice notice-info is-dismissible">';
		$notice .= '<p><b>:) We have spent countless hours developing this free plugin for you, and we would really appreciate it if you dropped us a quick rating. Your opinion matters a lot to us. It helps us to get better. Thanks for using PDF Invoice and Packing Slip WooCommerce.</b></p>';
		$notice .= '<ul>';
		$notice .= '<li><a val="later" href="#">Remind me later</a></li>';
		$notice .= '<li><a val="never" href="#">I would not</a></li>';
		$notice .= '<li><a val="given" href="#" target="_blank">Review Here</a></li>';
		$notice .= '</ul>';
		$notice .= '</div>';
		echo $notice;
	}else if((!array_key_exists('review_notice',$options)) || ($options['review_notice'] =='later' && time()>=($options['updated_at'] + (60*60*24*30) ))) {
		$notice = '<div class="woo-pdf-review-notice notice notice-info is-dismissible">';
		$notice .= '<p><b>:) We have spent countless hours developing this free plugin for you, and we would really appreciate it if you dropped us a quick rating. Your opinion matters a lot to us. It helps us to get better. Thanks for using PDF Invoice and Packing Slip WooCommerce.</b></p>';
		$notice .= '<ul>';
		$notice .= '<li><a val="later" href="#">Remind me later</a></li>';
		$notice .= '<li><a val="never" href="#">I would not</a></li>';
		$notice .= '<li><a val="given" href="#" target="_blank">Review Here</a></li>';
		$notice .= '</ul>';
		$notice .= '</div>';
		echo $notice;
	}
}



/**
 * Show Review request admin notice
 */
add_action('wp_ajax_save_review_notice', 'woo_pdf_save_review_notice');
function woo_pdf_save_review_notice() {
	$notice = sanitize_text_field($_POST['notice']);
	$value['review_notice'] = $notice;
	$value['updated_at'] = time();

	update_option('woo_pdf_review_notice',$value);
	wp_send_json_success($value);
}





